#include "Collision.h"

float Collision::distance(Vec2 p0, Vec2 p1)         //calculate distance between two Vec2 pos
{
	float dx = p1.x - p0.x;
	float dy = p1.y - p0.y;
	return (float)sqrt(dx *dx + dy * dy);
}

float Collision::distanceXY(float x0, float y0, float x1, float y1)     //calculates distance between two point pairs
{
	float dx = x1 - x0;
	float dy = y1 - y0;
	return (float)sqrt(dx *dx + dy * dy);
}

bool Collision::inRange(int value, int min, int max)   //calculates if a given value is within a certain range
{
	return value >= min && value <= max;
}

bool Collision::pointInRect(int x, int y, RectI  rect)  //calculates if a point is within the dimensions of a rectangle
{
	return inRange(x, rect.left, rect.right) &&
		inRange(y, rect.top, rect.bottom);
}

bool Collision::circleCollision(Vec2 c0, Vec2 c1, float c0Radius, float c1Radius)  //calculates if two circles collide
{
	return distance(c0, c1) <= c0Radius + c1Radius;
}

bool Collision::circlePointCollision(float x, float y, Vec2 circle, float radius)   //calculates if a point is within the dimensions of a circle
{
	return distanceXY(x, y, circle.x, circle.y) < radius;
}

Vec2 Collision::lineIntersect(Vec2 p0, Vec2 p1, Vec2 p2, Vec2 p3)    //returns a Vec2 pos where two line intersect
{
	float a1 = p1.y - p0.y;
	float b1 = p0.x - p1.x;
	float c1 = a1 * p0.x + b1 * p0.y;
	float a2 = p3.y - p2.y;
	float b2 = p2.x - p3.x;
	float c2 = a2 * p2.x + b2 * p2.y;
	float denominator = a1 * b2 - a2 * b1;

	float x = (b2 * c1 - b1 * c2) / denominator;
	float y = (a1 * c2 - a2 * c1) / denominator;
	return { x, y };
}
