#pragma once
#include "ChiliWin.h"
#include <d3d11.h>
#include <wrl.h>
#include "ChiliException.h"
#include "Colors.h"
#include "Surface.h"
#include "RectI.h"
#include <math.h>

class Graphics
{
public:
	class Exception : public ChiliException
	{
	public:
		Exception( HRESULT hr,const std::wstring& note,const wchar_t* file,unsigned int line );
		std::wstring GetErrorName() const;
		std::wstring GetErrorDescription() const;
		virtual std::wstring GetFullMessage() const override;
		virtual std::wstring GetExceptionType() const override;
	private:
		HRESULT hr;
	};
private:
	// vertex format for the framebuffer fullscreen textured quad
	struct FSQVertex
	{
		float x,y,z;		// position
		float u,v;			// texcoords
	};
public:
	Graphics( class HWNDKey& key );
	Graphics( const Graphics& ) = delete;
	Graphics& operator=( const Graphics& ) = delete;
	void EndFrame();
	void BeginFrame();
	void PutPixel( int x,int y,int r,int g,int b )
	{
		PutPixel( x,y,{ unsigned char( r ),unsigned char( g ),unsigned char( b ) } );
	}
	void PutPixel( int x,int y,Color c );
	void PutPixelClipped(int x, int y, Color c);
	void DrawSpriteNonChroma(int x, int y, const Surface& s);
	void DrawSpriteNonChroma(int x, int y, const RectI& srcRect, const Surface& s);
	void DrawSpriteNonChroma(int x, int y, RectI srcRect, const RectI& clip, const Surface& s);
	void DrawSprite(int x, int y, const Surface& s, Color chroma);
	void DrawSprite(int x, int y, const RectI& srcRect, const Surface& s, Color chroma );
	void DrawSprite(int x, int y, RectI srcRect, const RectI& clip, const Surface& s, Color chroma );

	void DrawSpriteSubstitute(int x, int y, const Surface& s,Color substitute, Color chroma);
	void DrawSpriteSubstitute(int x, int y, const RectI& srcRect, const Surface& s,Color substitute, Color chroma);
	void DrawSpriteSubstitute(int x, int y, RectI srcRect, const RectI& clip, const Surface& s,Color substitute, Color chroma);

	void DrawLine(float x1, float y1, float x2, float y2,Color c);
	void DrawLineClipped(float x1, float y1, float x2, float y2, Color c);

	void DrawRect(int xPos, int yPos, int width, int height, bool filled, Color c);
	void DrawRect(RectI rect, bool filled, Color c);
	void DrawBox(int xPos, int yPos, int dimension, Color c);
	void DrawCircle(int cx, int cy, int radius, Color c);
	void DrawDisc(int cx, int cy, int radius, Color c);
	void DrawEnergyWave(int centerX, int centerY, int radius, Color c);
	void DrawReticle(int x, int y, Color c);
	void DrawCoin(int x, int y, int radius, Color c);
	void DrawBoxGrid(int x, int y, int numBoxes, int dimension, int padding, Color c);
	void DrawGrid(Color c);

	static RectI GetScreenRect();

	~Graphics();
private:
	Microsoft::WRL::ComPtr<IDXGISwapChain>				pSwapChain;
	Microsoft::WRL::ComPtr<ID3D11Device>				pDevice;
	Microsoft::WRL::ComPtr<ID3D11DeviceContext>			pImmediateContext;
	Microsoft::WRL::ComPtr<ID3D11RenderTargetView>		pRenderTargetView;
	Microsoft::WRL::ComPtr<ID3D11Texture2D>				pSysBufferTexture;
	Microsoft::WRL::ComPtr<ID3D11ShaderResourceView>	pSysBufferTextureView;
	Microsoft::WRL::ComPtr<ID3D11PixelShader>			pPixelShader;
	Microsoft::WRL::ComPtr<ID3D11VertexShader>			pVertexShader;
	Microsoft::WRL::ComPtr<ID3D11Buffer>				pVertexBuffer;
	Microsoft::WRL::ComPtr<ID3D11InputLayout>			pInputLayout;
	Microsoft::WRL::ComPtr<ID3D11SamplerState>			pSamplerState;
	D3D11_MAPPED_SUBRESOURCE							mappedSysBufferTexture;
	Color*                                              pSysBuffer = nullptr;
public:
	static constexpr int ScreenWidth = 800;
	static constexpr int ScreenHeight = 600;
};