#include "Player.h"

void Player::ClampToScreen()
{
	const float right = x + width;
	if (x - width < 1)
	{
		x = width;
	}
	else if (right >= float(Graphics::ScreenWidth))
	{
		x = float(Graphics::ScreenWidth - 1) - width;
	}

	const float bottom = y + height;
	if (y < 0)
	{
		y = 0;
	}
	else if (bottom >= float(Graphics::ScreenHeight))
	{
		y = float(Graphics::ScreenHeight - 1) - height;
	}
}

Player::Player()
	:
	x(400.0f),
	y(300.0f),
	movementAngle(3 * PI / 2),
	angle(3 * PI / 2),
	speedX(0.0f),
	speedY(0.0f),
	gravity(1.5f)
{
}

void Player::Draw(Graphics & gfx) const
{
	//gfx.DrawShip(x - cameraX, y - cameraY, angle);
}

void Player::Update(const Keyboard & kbd)
{
	ClampToScreen();
	UpdateCamera((int)x, (int)y);
	Controls(kbd);
}

void Player::Controls(const Keyboard & kbd)
{
	angle = 0.5f;
	
	/*if (kbd.KeyIsPressed(VK_RIGHT))
		angle += 0.06f;
	if (kbd.KeyIsPressed(VK_LEFT))
		angle -= 0.06f;
	if (angle > 2 * PI)
		angle = 0;*/

	if (kbd.KeyIsPressed(VK_UP))
	{
		float sumx = speedX + (0.07f * cos(angle));
		float sumy = speedY + (0.07f * sin(angle));

		movementAngle = atan2(sumy, sumx);
		if (movementAngle < 0.0f)
			movementAngle += 2 * PI;

		speedX += 0.07f * cos(angle);
		speedY += 0.07f * sin(angle);
	}
	else if(kbd.KeyIsPressed(VK_DOWN))
	{
		float sumx = speedX - (0.07f * cos(angle));
		float sumy = speedY - (0.07f * sin(angle));

		movementAngle = atan2(sumy, sumx);
		if (movementAngle < 0.0f)
			movementAngle -= 2 * PI;

		speedX -= 0.07f * cos(angle);
		speedY -= 0.07f * sin(angle);
	}
	else
	{
		if (speedX != 0)
			speedX += 0.02f * cos(movementAngle + PI);

		if (speedY != 0)
			speedY += 0.02f * sin(movementAngle + PI);

		if (speedX < 0.1f && speedX > -0.1f)
			speedX = 0;

		if (speedY < 0.1f && speedY > -0.1f)
			speedY = 0;
	}

	if (movementAngle > 0 && movementAngle < PI / 2 || movementAngle >(3 * PI / 2) && movementAngle < 2 * PI)
	{
		if (speedX > 4.0f * cos(movementAngle))
			speedX = 4.0f * cos(movementAngle);
	}
	else
	{
		if (abs(speedX) > -4.0f * cos(movementAngle))
			speedX = 4.0f * cos(movementAngle);
	}

	if (movementAngle > 0 && movementAngle < PI)
	{
		if (speedY > 4.0f * sin(movementAngle))
			speedY = 4.0f * sin(movementAngle);
	}
	else
	{
		if (abs(speedY) > -4.0f * sin(movementAngle))
			speedY = 4.0f * sin(movementAngle);
	}

	x += speedX;
	y += speedY;
}

void Player::UpdateCamera(int playerX, int playerY)
{
	// convert player world coordinates to screen coordinates
	int px = playerX - cameraX;
	int py = playerY - cameraY;
	int ScreenWidth = 800;
	int ScreenHeight = 600;

	// scroll camera if player is out of tracking rectangle
	if (px < CTRKOFFHOR)
	{
		cameraX = playerX - CTRKOFFHOR;
	}
	if (px + width > ScreenWidth - CTRKOFFHOR)
	{
		cameraX = playerX + (int)width + CTRKOFFHOR - ScreenWidth;
	}
	if (py < CTRKOFFVERT)
	{
		cameraY = playerY - CTRKOFFVERT;
	}
	if (py + height > ScreenHeight - CTRKOFFVERT)
	{
		cameraY = playerY + (int)height + CTRKOFFVERT - ScreenHeight;
	}

	// clamp camera so that the screen does not exceed
	// the edge of the world
	if (cameraX < 0)
	{
		cameraX = 0;
	}
	else if (cameraX + ScreenWidth > WORLDWIDTH)
	{
		cameraX = WORLDWIDTH - ScreenWidth;
	}
	if (cameraY < 0)
	{
		cameraY = 0;
	}
	else if (cameraY + ScreenHeight > WORLDHEIGHT)
	{
		cameraY = WORLDHEIGHT - ScreenHeight;
	}
}



float Player::GetPosX() const
{
	return x;
}

float Player::GetPosY() const
{
	return y;
}

float Player::GetWidth() const
{
	return width;
}

float Player::GetHeight() const
{
	return height;
}

int Player::GetCameraX() const
{
	return cameraX;
}

int Player::GetCameraY() const
{
	return cameraY;
}
