#include "Tools.h"
#include <stdlib.h>
using namespace std;

Tools::Tools()
	:
	keyPressedLastFrame(false),
	SaveCoords(false)
{
	rects.reserve(NUMRECTS);
	DefineButtons();

	grid.multiplier = 100;
	grid.GridLocked = false;
	text.x = 723;
	text.y = 13;

	grid.color = Colors::Red;
	text.color = Colors::Green;

	settings.IconX = 25.0f;
	settings.IconY = 550.0f;
	settings.Width = 33.0f;
	settings.Height = 33.0f;
	settings.isActive = false;
	settings.DrawGrid = true;
	settings.ShowCoords = true;
	settings.useAltControls = false;

	for (int i = 0; i < MAXGRIDFILL; i++)
	{
		gridFill.x[i] = NULL;
		gridFill.y[i] = NULL;
		gridFill.fill[i] = false;
		gridFill.countArray[i] = i + 1;
	}

	for (int i = 0; i < MAXMARKPOINTS; i++)
	{
		PointIE.x[i] = NULL;
		PointIE.y[i] = NULL;
		PointIE.isDrawn[i] = false;
	}

	gridFill.count = 0;
	ie.Import = false;
	ie.Export = false;
	PointIE.SavePoints = false;
	PointIE.Import = false;
	PointIE.Export = false;
	PointIE.count = 0;
	PointIE.color = Colors::White;
}

/***************************DRAWING**********************************************/

void Tools::Draw(Graphics & gfx, Mouse & mouse)
{       

	if (!settings.isActive)
	{
		FillGridSquare(gfx);
		DrawPoints(gfx);

		if (settings.DrawGrid)
		{
			DrawGrid(grid.multiplier, gfx, grid.color);
		}

		if (settings.ShowCoords)
		{

			DrawCoordinates(text.x, text.y, text.color, gfx);
		}

		gfx.DrawSpriteNonChroma((int)settings.IconX, (int)settings.IconY, SettingsIcon);
		gfx.DrawSprite(746, 550, RefreshIcon, Colors::White);
	}
	
	if (settings.isActive)
	{
		DrawSettingsMenu(gfx, mouse);
	}
}

void Tools::FillGridSquare(Graphics & gfx)
{
	int dimension = grid.multiplier;

	for (int i = 0; i < MAXGRIDFILL; i++)
	{
		if (gridFill.fill[i])
		{
			grid.GridLocked = true;

			if (gridFill.count <= MAXGRIDFILL)
			{
				count = to_string(gridFill.countArray[i]);
			}

			FilledGridX = to_string(gridFill.x[i]);
			FilledGridY = to_string(gridFill.y[i]);
			FilledGridWidth = to_string(gridFill.x[i] + grid.multiplier);
			FilledGridHeight = to_string(gridFill.y[i] + grid.multiplier);

			gfx.DrawRect(gridFill.x[i], gridFill.y[i], dimension, dimension,true, Colors::Blue);

			if (grid.multiplier > 19)
			{
				fontSmall.DrawText(count, { gridFill.x[i] + 5, gridFill.y[i] + 5 }, gfx, Colors::Green);
			}
			else
			{
				fontSmall.DrawText(count, { gridFill.x[i], gridFill.y[i] }, gfx, Colors::Green);
			}
		}
	}
}

void Tools::DrawGrid(int multiplier, Graphics & gfx, Color c)
{
	for (int yy = multiplier; yy < 600; yy += multiplier)
	{
		gfx.DrawLineClipped(0.0f, (float)yy, 800.0f, (float)yy, c);  //draws horizontal lines
	}

	for (int xx = multiplier; xx < 800; xx += multiplier)
	{
		gfx.DrawLineClipped((float)xx, 0.0f, (float)xx, 600.0f, c);  //draws vertical lines
	}
}

void Tools::DrawPoints(Graphics & gfx)
{
	for (int i = 0; i < MAXMARKPOINTS; i++)
	{
		if (PointIE.isDrawn[i])
		{
			gfx.DrawReticle(PointIE.x[i], PointIE.y[i], PointIE.color);
		}
	}
}

void Tools::DrawCoordinates(int x, int y, Color c, Graphics& gfx)
{
	fontSmall.DrawText(MXCoords, { x,y }, gfx, c);
	fontSmall.DrawText(MYCoords, { x + 30,y }, gfx, c);
	fontSmall.DrawText("Multiplier", { 708, 35 }, gfx, c);
	fontSmall.DrawText(CurrentMultiplier, { 738, 55 }, gfx, c);
}

void Tools::DrawSettingsMenu(Graphics & gfx, Mouse& mouse)
{
	if (settings.isActive)
	{
		
		int x = 35;
		int y = 72;

		if (settings.DrawGrid)
		{
			gfx.DrawRect(202, 72, 27, 25, true,Colors::Blue);
		}
		else
		{
			gfx.DrawRect(235, 72, 27, 25,true, Colors::Blue);
		}

		if (settings.ShowCoords)
		{
			gfx.DrawRect(202, 125, 27, 26,true, Colors::Blue);
		}
		else
		{
			gfx.DrawRect(235, 125, 27, 26, true, Colors::Blue);
		}

		if (settings.useAltControls)
		{
			gfx.DrawRect(389, 391, 28, 26, true, Colors::Blue);
		}
		else
		{
			gfx.DrawRect(422, 391, 28, 26, true, Colors::Blue);
		}

		gfx.DrawSprite(20, 15, BackIcon, Colors::White);

		fontLarge.DrawText("Settings And Help", { 255,15 }, gfx, Colors::White);

		fontMedium.DrawText("Grid ON/OFF", { x,y }, gfx, Colors::White);
		gfx.DrawRect(rects[gridOn], false, Colors::White);
		fontSmall.DrawText("ON", { 206,76 }, gfx, Colors::White);
		gfx.DrawRect(rects[gridOff], false, Colors::White);
		fontSmall.DrawText("OFF", { 237,76 }, gfx, Colors::White);

		fontMedium.DrawText("Show Coords", { x, y + 53 }, gfx, Colors::White);
		gfx.DrawRect(rects[coordsOn], false, Colors::White);
		fontSmall.DrawText("ON", { 206,129 }, gfx, Colors::White);
		gfx.DrawRect(rects[coordsOff], false, Colors::White);
		fontSmall.DrawText("OFF", { 237,129 }, gfx, Colors::White);

		fontMedium.DrawText("Marked Grid Squares", { x, y + 106 }, gfx, Colors::White);
		fontMedium.DrawText("Save", { x + 354, y + 106 }, gfx, Colors::White);
		fontMedium.DrawText("Import", { x + 432, y + 106 }, gfx, Colors::White);
		fontMedium.DrawText("Export", { 577, y + 106 }, gfx, Colors::White);

		fontMedium.DrawText("Marked Points", { x, y + 160 }, gfx, Colors::White);
		fontMedium.DrawText("Save", { x + 354, y + 160 }, gfx, Colors::White);
		fontMedium.DrawText("Import", { x + 432, y + 160 }, gfx, Colors::White);
		fontMedium.DrawText("Export", { 577, y + 160 }, gfx, Colors::White);

		fontMedium.DrawText("Save/Export All", { x + 354, y + 204 }, gfx, Colors::White);

		fontMedium.DrawText("Import All", { x + 354, y + 248 }, gfx, Colors::White);

		fontMedium.DrawText("Use Alternate Controls", { x, 392 }, gfx, Colors::White);
		gfx.DrawRect(rects[altOn], false, Colors::White);
		fontSmall.DrawText("ON", { x + 359, 396 }, gfx, Colors::White);
		gfx.DrawRect(rects[altOff], false, Colors::White);
		fontSmall.DrawText("OFF", { x + 390, 395 }, gfx, Colors::White);

	}
}

/***************************LOGIC**********************************************/

void Tools::Update(Graphics & gfx, Mouse & mouse, Keyboard & kbd)
{
	MXCoords = to_string(mouse.GetPosX());  //convert mouse position to string
	MYCoords = to_string(mouse.GetPosY());
	CurrentMultiplier = to_string(grid.multiplier);  //convert the current grid multiplier to string 

	SettingsMenuLogic(mouse, gfx);
	OutputFilledGridCoords();
	ExportFilledGridCoords();
	ImportFilledGridCoords();
	OutputMarkedPoints();
	ExportMarkedPoints();
	ImportMarkedPoints();
	if (!settings.isActive)
	{

		Reset(mouse, gfx);
		Controls(kbd, mouse);
	}

	if (grid.multiplier > grid.maxMultiplier)
	{
		grid.multiplier = grid.maxMultiplier;
	}

	if (grid.multiplier < grid.minMultiplier)
	{
		grid.multiplier = grid.minMultiplier;
	}
}

bool Tools::getSettingsActiveStatus()
{
	return settings.isActive;
}

void Tools::Controls(Keyboard & kbd, Mouse& mouse)
{
	if (!grid.GridLocked)
	{
		if (kbd.KeyIsPressed(VK_UP) && grid.multiplier <= grid.maxMultiplier)
		{
			grid.multiplier++;
		}

		if (kbd.KeyIsPressed(VK_DOWN) && grid.multiplier >= grid.minMultiplier)
		{
			grid.multiplier--;
		}
	}



	if (!keyPressedLastFrame)
	{
		if (kbd.KeyIsPressed(VK_SPACE))
		{
			if (PointIE.count < MAXMARKPOINTS)
			{
				keyPressedLastFrame = true;
				PointIE.isDrawn[PointIE.count] = true;
				PointIE.x[PointIE.count] = mouse.GetPosX();
				PointIE.y[PointIE.count] = mouse.GetPosY();
				PointIE.count++;
			}
		}
		if (!grid.GridLocked)
		{
			if (kbd.KeyIsPressed(VK_RIGHT) && grid.multiplier < grid.maxMultiplier)
			{
				grid.multiplier += 5;

				keyPressedLastFrame = true;
			}
			if (kbd.KeyIsPressed(VK_LEFT))
			{
				keyPressedLastFrame = true;

				if (grid.multiplier >= 5)
				{
					grid.multiplier -= 5;
				}
			}
		}



		if (mouse.RightIsPressed())
		{
			keyPressedLastFrame = true;

			if (gridFill.count < MAXGRIDFILL)
			{
				gridFill.fill[gridFill.count] = true;

				gridFill.x[gridFill.count] = mouse.GetPosX() - (mouse.GetPosX() % grid.multiplier);
				gridFill.y[gridFill.count] = mouse.GetPosY() - (mouse.GetPosY() % grid.multiplier);

				gridFill.count += 1;


				grid.GridLocked = true;
			}

		}


	}

	if (!kbd.KeyIsPressed(VK_SPACE) && !mouse.RightIsPressed() && !kbd.KeyIsPressed(VK_LEFT) && !kbd.KeyIsPressed(VK_RIGHT))
	{

		keyPressedLastFrame = false;
	}
}

void Tools::AltControls(Keyboard & kbd, Mouse & mouse)
{
}

void Tools::SettingsMenuLogic(Mouse & mouse, Graphics & gfx)
{
	int mx = mouse.GetPosX();
	int my = mouse.GetPosY();
	if (!settings.isActive)
	{
		
		if (collision.pointInRect(mx,my,rects[settingsButton]))
		{
			gfx.DrawRect(settings.IconX - 1, settings.IconY - 1, settings.Width, settings.Height,false, Colors::Green);
			fontSmall.DrawText("Settings", { 18, 530 }, gfx, Colors::White);

			if (mouse.LeftIsPressed())
			{
				settings.isActive = true;                    //settings icon on main screen
			}
		}
	}

	if (settings.isActive)
	{
		if (collision.pointInRect(mx, my, rects[backButton]))
		{
			gfx.DrawRect(rects[backButton], false, Colors::Green);
			fontSmall.DrawText("Back", { 25, 6 }, gfx, Colors::White);

			if (mouse.LeftIsPressed())
			{
				settings.isActive = false;
			}
		}

		if (collision.pointInRect(mx, my, rects[gridOn]))
		{
			if (mouse.LeftIsPressed())
			{
				settings.DrawGrid = true;                //on button for drawing the grid
			}
		}
		else if (collision.pointInRect(mx, my, rects[gridOff]))
		{
			if (mouse.LeftIsPressed())
			{
				settings.DrawGrid = false;                  //off button for drawing the grid
			}
		}

		if (collision.pointInRect(mx, my, rects[coordsOn]))
		{
			if (mouse.LeftIsPressed())
			{
				settings.ShowCoords = true;           //on button for showing coords
			}
		}
		else if (collision.pointInRect(mx, my, rects[coordsOff]))
		{
			if (mouse.LeftIsPressed())
			{
				settings.ShowCoords = false;              //off button for showing coords
			}
		}

		if (collision.pointInRect(mx, my, rects[altOn]))
		{
			if (mouse.LeftIsPressed())
			{
				settings.useAltControls = true;
			}
		}
		else if (collision.pointInRect(mx, my, rects[altOff]))
		{
			if (mouse.LeftIsPressed())
			{
				settings.useAltControls = false;
			}
		}

		if (collision.pointInRect(mx, my, rects[gridSave]))
		{
			gfx.DrawRect(rects[gridSave], false, Colors::Green);    //save marked squares
			if (mouse.LeftIsPressed())
			{
				SaveCoords = true;
			}
		}
		else if (collision.pointInRect(mx, my, rects[gridImport]))
		{
			gfx.DrawRect(rects[gridImport], false, Colors::Green);     //import marked square data
			if (mouse.LeftIsPressed())
			{
				ie.Import = true;
			}
		}
		else if (collision.pointInRect(mx, my, rects[gridExport]))
		{
			gfx.DrawRect(rects[gridExport], false, Colors::Green);     //export marked square data
			if (mouse.LeftIsPressed())
			{
				ie.Export = true;
			}
		}
		
		if (collision.pointInRect(mx,my,rects[pointSave]))
		{
			gfx.DrawRect(rects[pointSave], false, Colors::Green);    //save marked points
			if (mouse.LeftIsPressed())
			{
				PointIE.SavePoints = true;
			}
		}
		else if (collision.pointInRect(mx, my, rects[pointImport]))
		{
			gfx.DrawRect(rects[pointImport], false, Colors::Green);     //import marked points data
			if (mouse.LeftIsPressed())
			{
				PointIE.Import = true;
			}
		}
		else if (collision.pointInRect(mx, my, rects[pointExport]))
		{
			gfx.DrawRect(rects[pointExport], false, Colors::Green);      //export marked points data
			if (mouse.LeftIsPressed())
			{
				PointIE.Export = true;
			}
		}

		if (collision.pointInRect(mx, my, rects[exportAll]))
		{
			gfx.DrawRect(rects[exportAll], false, Colors::Green);   //export and save all
			if (mouse.LeftIsPressed())
			{
				SaveCoords = true;
				ie.Export = true;
				PointIE.SavePoints = true;
				PointIE.Export = true;
			}
		}

		if (collision.pointInRect(mx, my, rects[importAll]))
		{
			gfx.DrawRect(rects[importAll], false, Colors::Green);    //import all
			if (mouse.LeftIsPressed())
			{
				ie.Import = true;
				PointIE.Import = true;
			}
		}
	}
}

void Tools::Reset(Mouse & mouse, Graphics & gfx)
{
	int mx = mouse.GetPosX();
	int my = mouse.GetPosY();
	if (collision.pointInRect(mx,my,rects[resetButton]))
	{
		gfx.DrawRect(rects[resetButton],false, Colors::Green);
		fontSmall.DrawText("Reset", { 740,530 }, gfx, Colors::White);
		
		if (mouse.LeftIsPressed())
		{
			grid.multiplier = 100;
			grid.GridLocked = false;
			settings.isActive = false;
			keyPressedLastFrame = false;
			gridFill.count = NULL;
			PointIE.count = NULL;

			for (int i = 0; i < MAXGRIDFILL; i++)
			{
				gridFill.x[i] = NULL;
				gridFill.y[i] = NULL;
				gridFill.fill[i] = false;
				PointIE.x[i] = NULL;
				PointIE.y[i] = NULL;
				PointIE.isDrawn[i] = false;
			}

			
		}
	}
}

void Tools::DefineButtons()
{
	rects[gridOn] =         { 201, 229,  71,  97 };
	rects[gridOff] =        { 234, 262,  71,  97 };
	rects[coordsOn] =       { 201, 229, 124, 150 };
	rects[coordsOff] =      { 234, 262, 124, 150 };
	rects[gridSave] =       { 389, 440, 178, 202 };
	rects[gridImport] =     { 467, 544, 178, 202 };
	rects[gridExport] =     { 577, 655, 178, 202 };
	rects[pointSave] =      { 389, 440, 232, 256 };
	rects[pointImport] =    { 467, 544, 232, 256 };
	rects[pointExport] =    { 577, 655, 232, 256 };
	rects[exportAll] =      { 389, 584, 276, 300 };
	rects[importAll] =      { 389, 519, 320, 344 };
	rects[altOn] =          { 389, 417, 391, 417 };
	rects[altOff] =         { 422, 450, 391, 417 };
	rects[settingsButton] = { 25,   56, 549, 582 };
	rects[backButton] =     { 25,   56,  21,  49 };
	rects[resetButton] =    { 745, 774, 549, 581 };
}


/***************************OUTPUT**********************************************/

void Tools::OutputFilledGridCoords()
{
	if (SaveCoords)
	{
		writeFile.open(ie.SavePath);
		writeFile << "The coordinates are listed in the order in which you marked the grid squares. \n";
		writeFile << "The number of the marked square corresponds with the number next to coordinates below. \n";
		writeFile << '\n';

		for (int i = 0; i < MAXGRIDFILL; i++)
		{
			if (gridFill.fill[i])
			{
				int newCount = i + 1;
				int x2 = gridFill.x[i] + grid.multiplier;
				int y2 = gridFill.y[i] + grid.multiplier;
				
				writeFile << newCount << ") X1: " << gridFill.x[i] << " Y1: " << gridFill.y[i] << " X2: " << x2 << " Y2: " << y2 << endl;
			}
		}
		SaveCoords = false;
		writeFile.close();
	}
}

void Tools::ExportFilledGridCoords()
{
	if (ie.Export)
	{
		
		writeFile.open(ie.CountPath);
		writeFile << grid.multiplier << endl;
		writeFile << gridFill.count << endl;
		writeFile.close();

		writeFile.open(ie.PosPath);

		for (int i = 0; i < MAXGRIDFILL; i++)
		{
			if (gridFill.fill[i])
			{
				
				writeFile << gridFill.x[i] << endl;
				writeFile << gridFill.y[i] << endl;
			}
		}

		ie.Export = false;
		writeFile.close();
	}
}

void Tools::ImportFilledGridCoords()
{
	if (ie.Import)
	{
		readFile.open(ie.CountPath);
		readFile >> grid.multiplier >> gridFill.count;
		readFile.close();
		
		readFile.open(ie.PosPath);

		for (int i = 0; i < gridFill.count; i++)
		{
			readFile >> gridFill.x[i] >> gridFill.y[i];
			gridFill.fill[i] = true;
		}
	
		ie.Import = false;
		readFile.close();
	}
	
}

void Tools::OutputMarkedPoints()
{
	if (PointIE.SavePoints)
	{
		writeFile.open(PointIE.SavePath);
		
		for (int i = 0; i < MAXMARKPOINTS; i++)
		{
			if (PointIE.isDrawn[i])
			{
				writeFile << "X: " << PointIE.x[i] << " Y: " << PointIE.y[i] << endl;
			}
		}

		PointIE.SavePoints = false;
		writeFile.close();
	}
}

void Tools::ExportMarkedPoints()
{
	if (PointIE.Export)
	{
		writeFile.open(PointIE.CountPath);
		writeFile << PointIE.count;
		writeFile.close();

		writeFile.open(PointIE.PointPosPath);
		{
			for (int i = 0; i < PointIE.count; i++)
			{
				if (PointIE.isDrawn[i])
				{
					writeFile << PointIE.x[i] << endl;
					writeFile << PointIE.y[i] << endl;
				}
			}
		}
		writeFile.close();
		PointIE.Export = false;
	}
}

void Tools::ImportMarkedPoints()
{
	if (PointIE.Import)
	{
		readFile.open(PointIE.CountPath);
		readFile >> PointIE.count;
		readFile.close();
		readFile.open(PointIE.PointPosPath);
		for (int i = 0; i < PointIE.count; i++)
		{
			readFile >> PointIE.x[i] >> PointIE.y[i];
			PointIE.isDrawn[i] = true;
		}
		readFile.close();
		PointIE.Import = false;
	}
}



