#pragma once
#include"Graphics.h"
#include "Font.h"
#include "Mouse.h"
#include "Keyboard.h"
#include "Surface.h"
#include "Colors.h"
#include <string>
#include <iostream>
#include <fstream>
#include <vector>
#include "Vec2.h"
#include "Collision.h"
#include "RectI.h"


using namespace std;

class Tools
{

public:
	Tools();
	void Draw(Graphics& gfx, Mouse& mouse);
	void Update(Graphics& gfx, Mouse& mouse, Keyboard& kbd);
	bool getSettingsActiveStatus();
private:
	/***************************DRAWING**********************************************/

	void FillGridSquare(Graphics& gfx);
	void DrawGrid(int multiplier,Graphics& gfx, Color c);
	void DrawPoints(Graphics& gfx);
	void DrawCoordinates(int x, int y, Color c, Graphics& gfx);
	void DrawSettingsMenu(Graphics& gfx, Mouse& mouse);
	
	/***************************LOGIC**********************************************/

	void SettingsMenuLogic(Mouse& mouse, Graphics& gfx);
	void Controls(Keyboard& kbd, Mouse& mouse);
	void AltControls(Keyboard& kbd, Mouse& mouse);
	void Reset(Mouse& mouse, Graphics& gfx);
	void DefineButtons();
	
	/***************************OUTPUT**********************************************/

	void OutputFilledGridCoords();
	void ExportFilledGridCoords();
	void ImportFilledGridCoords();

	void OutputMarkedPoints();
	void ExportMarkedPoints();
	void ImportMarkedPoints();

	/***************************SPRITES**********************************************/

	Surface SettingsIcon = "Sprites\\SettingsIcon.bmp";
	Surface RefreshIcon = "Sprites\\RefreshIcon.bmp";
	Surface BackIcon = "Sprites\\Back42x37.bmp";
	Font fontSmall = "Sprites\\Fixedsys8x15.bmp";
	Font fontMedium = "Sprites\\Consolas13x24.bmp";
	Font fontLarge = "Sprites\\Fixedsys16x28.bmp";
	
	/***************************VARIABLES**********************************************/

	string MXCoords;
	string MYCoords;
	string CurrentMultiplier;
	string FilledGridX;
	string FilledGridY;
	string FilledGridWidth;
	string FilledGridHeight;
	string count;
	
	ofstream writeFile;
	ifstream readFile;

	static constexpr int MAXGRIDFILL = 50;
	static constexpr int MAXMARKPOINTS = 50;
	static constexpr int NUMRECTS = 17;

	vector <RectI> rects;
	bool keyPressedLastFrame;
	bool SaveCoords;

	struct GRID
	{
		int x, y, multiplier;
		const int minMultiplier = 10;
		const int maxMultiplier = 200;
		bool GridLocked;
		Color color;
	};

	struct TEXT
	{
		int x, y;
		Color color;
	};

	struct SETTINGS
	{
		float IconX, IconY, Width, Height;
		bool isActive;
		bool DrawGrid;
		bool ShowCoords;
		bool useAltControls;
	};

	struct GRIDFILL
	{
		int x[MAXGRIDFILL];
		int y[MAXGRIDFILL];
		bool fill[MAXGRIDFILL];
		int count; 
		int countArray[MAXGRIDFILL];
	};

	struct IMPORTEXPORT
	{
		bool Import;
		bool Export;
		const string SavePath = "Coordinates\\GridCoords.txt";
		const string CountPath = "ImportExport\\MulitplierAndCount.txt";
		const string PosPath = "ImportExport\\RawLocationData.txt";
	};

	struct POINTIE
	{
		int x[MAXMARKPOINTS];
		int y[MAXMARKPOINTS];
		bool isDrawn[MAXMARKPOINTS];
		int count;
		bool SavePoints;
		bool Import;
		bool Export;
		Color color;
		const string SavePath = "Coordinates\\PointCoords.txt";
		const string CountPath = "ImportExport\\PointCount.txt";
		const string PointPosPath = "ImportExport\\PointPos.txt";
	};

	enum BUTTONS
	{
		gridOn,
		gridOff,
		coordsOn,
		coordsOff,
		gridSave,
		gridExport,
		gridImport,
		pointSave,
		pointExport,
		pointImport,
		exportAll,
		importAll,
		altOn,
		altOff,
		settingsButton,
		backButton,
		resetButton

	};

	GRID grid;
	TEXT text;
	SETTINGS settings;
	GRIDFILL gridFill;
	IMPORTEXPORT ie;
	POINTIE PointIE;
	Collision collision;
};

